//////////////////////////////////////////////////////////////////////////
function GBWebPageIterator()
{
	this.dataPartsDivier = "<GBDiv>";
	this.outputDivider = "<GBOutputDiv>";
	this.attributesDivider = "<GBAttribute>"
	this.attributePartDivider = "<GBEquals>";
			
	this.currentDocument = document;
	
	this.currentStyleSheets = this.currentDocument.styleSheets;
	this.currentStyleSheetIndex = 0;
	this.currentRuleIndex = 0;
	
	this.currentElement = null;
	this.currentElementFinished = false;

	this.dynamicFrameNumber = 1;
	this.documentsStack = new Array();
}

//////////////////////////////////////////////////////////////////////////
//Document info methods
GBWebPageIterator.prototype.setCurrentDocument = function(anEnclosingFrameID)
{
	var theResult = "false";
	
	if (anEnclosingFrameID)
	{
		var theFrameElement = this.currentDocument.getElementById(anEnclosingFrameID);
		var theNewDocument = theFrameElement.contentDocument;
		if (theNewDocument)
		{
			this.documentsStack.push(this.currentDocument);
			this.currentDocument = theNewDocument;
			theResult = "true";
		}
	}
	else
	{
		var theDocument = this.documentsStack.pop();
		if (!theDocument)
		{
			theDocument = document;
		}
		
		this.currentDocument = theDocument;
		theResult = "true";
	}
		
	this.currentStyleSheets = this.currentDocument.styleSheets;
	this.currentStyleSheetIndex = 0;
	this.currentRuleIndex = 0;
	
	this.currentElement = null;
	this.currentElementFinished = false;
	
	return theResult;
}

GBWebPageIterator.prototype.reset = function()
{
	this.currentDocument = document;
	this.documentsStack = new Array();
	
	this.currentStyleSheets = this.currentDocument.styleSheets;
	this.currentStyleSheetIndex = 0;
	this.currentRuleIndex = 0;
	
	this.currentElement = null;
	this.currentElementFinished = false;
}

GBWebPageIterator.prototype.documentTitle = function()
{
	return this.currentDocument.title;
}

GBWebPageIterator.prototype.documentType = function()
{
	var theDocType = this.currentDocument.doctype;
	if (!theDocType)
	{
		return "";
	}
	
	var theDocumentTypeString = theDocType.name;
	if (!theDocumentTypeString)
	{
		theDocumentTypeString = "HTML";
	}
	
	if (theDocType.publicId)
	{
		theDocumentTypeString += " PUBLIC \"" + theDocType.publicId + "\"";
	}
	
	if (theDocType.systemId)
	{
		theDocumentTypeString += " \"" + theDocType.systemId + "\"";
	}
	
	return theDocumentTypeString;
}

GBWebPageIterator.prototype.documentURL = function()
{
	return this.currentDocument.documentURI;
}

GBWebPageIterator.prototype.documentBaseURL = function()
{
	return this.currentDocument.baseURI;
}	

//////////////////////////////////////////////////////////////////////////
// Elements enumeration
GBWebPageIterator.prototype.adjustedValueForAttribute = function(anElement, anAttribute)
{
	var theName = anAttribute.name;
	if (theName)
	{
		theName = theName.toLowerCase();
	
		if (theName == "src")
		{
			if (anElement.src)
			{
				return anElement.src;
			}	
		}
		else if (theName == "style")
		{
			return null;
		}
		else if (theName == "onload" || theName == "onclick" || theName == "onmousedown" 
					|| theName == "onmouseup" || theName == "onmouseover" 
					|| theName == "onmouseout" || theName == "onmousemove")
		{
			return null;
		}
	}

	return anAttribute.value;
}

GBWebPageIterator.prototype.formatElementAttributes = function(anElement)
{
	var theAttributes = anElement.attributes;
	if (!theAttributes)
	{
		return "";
	}
	
	var theResult = "";
	for (var i = 0; i < theAttributes.length; i++)
	{
		var theAttribute = theAttributes[i];
		var theName = theAttribute.name;
		var theValue = this.adjustedValueForAttribute(anElement, theAttribute); 
				
		if (theName && theName.length > 0 && typeof(theValue) != "undefined" && theValue != null)
		{
			if (theResult.length > 0)
			{
				theResult += this.attributesDivider;
			}
			
			theResult += (theName + this.attributePartDivider + theValue);
		}	
	}
	
	return theResult;
}

GBWebPageIterator.prototype.formatElementStyle = function(anElement)
{
	var theStyle = anElement.style;
	
	var theResult = null;
	if (theStyle)
	{
		theResult = theStyle.cssText;
	}
	
	if (!theResult)
	{
		theResult = "";
	}
	
	return theResult;
}

GBWebPageIterator.prototype.formatElement = function(anElement, aContainerFlag)
{
	// Format text nodes
	if (3 == anElement.nodeType)
	{
		return "singleElement:"  + this.dataPartsDivier + "#text" + 
					this.dataPartsDivier + this.dataPartsDivier + this.dataPartsDivier 
					+ anElement.textContent;
	}
	
	var theTagName = anElement.tagName.toLowerCase();
	
	if (theTagName == "frame" || theTagName == "iframe")
	{
		if (!anElement.id || 0 == anElement.id.length)
		{
			anElement.id = "GBFrame_" + this.dynamicFrameNumber;
			this.dynamicFrameNumber ++;
		}
	}
	
	var thePrefix = aContainerFlag ? "beginContainerElement:" : "singleElement:";
	
	var theAttributes = this.formatElementAttributes(anElement);
	var theStyle = this.formatElementStyle(anElement);
	
	var theResult = thePrefix + this.dataPartsDivier + anElement.tagName + this.dataPartsDivier + 
				theAttributes + this.dataPartsDivier + theStyle;
	
	if (!aContainerFlag)
	{
		var theInnerText = anElement.innerText;
		if (theInnerText && theInnerText.length > 0)
		{
			theResult += (this.dataPartsDivier + theInnerText);
		}
	}	
	
	return theResult;
}

GBWebPageIterator.prototype.formatEndOfElement = function(anElement)
{
	return "endContainerElement:" + this.dataPartsDivier + anElement.tagName;
}

GBWebPageIterator.prototype.beginElementsIteration = function()
{
	this.currentElement = this.currentDocument.body;
	
	this.currentElementFinished = false;
	return "beginOfElementsIteration";
}

GBWebPageIterator.prototype.shouldSkipElement = function(anElement)
{
	// Skip comments
	if (8 == anElement.nodeType)
	{
		return true;
	}
	
	// For text nodes skip element with whitespace text
	if (3 == anElement.nodeType || 8 == anElement.nodeType)
	{
		var theText = anElement.textContent;
		return null == theText.match(/[\S]+/);
	}
	
	// For element nodes scip scripts
	var theTagName = anElement.tagName;
	return (theTagName && theTagName.toLowerCase() == "script");
}

GBWebPageIterator.prototype.nextElement = function()
{
	var theGoDeeperFlag = false;
	
	// If body is finished exit
	if (this.currentElement == this.currentDocument.body.parentNode)
	{
		return "endOfElementsIteration";
	}
	
	var theResult = "";
	// Determine if we should skip element
	var theSkipFlag = this.shouldSkipElement(this.currentElement);

	if (!theSkipFlag)
	{
		if (!this.currentElementFinished)
		{
			// Go deeper for elements that has children elements. Note that
			// for elements with text children only we will use innerText to get
			// text value and will not go into
			theGoDeeperFlag = this.currentElement.children && 
					this.currentElement.children.length > 0;
			
			theResult = this.formatElement(this.currentElement, theGoDeeperFlag);		
		}
		else
		{
			theResult = this.formatEndOfElement(this.currentElement);
		}
	}
	
	if (theGoDeeperFlag)
	{
		this.currentElement = this.currentElement.firstChild;
		this.currentElementFinished = false;
	}
	else
	{
		var theNextSibling = this.currentElement.nextSibling;
	
		if (!theNextSibling)
		{
			this.currentElement = this.currentElement.parentNode;
			this.currentElementFinished = true;
		}
		else
		{
			this.currentElement = theNextSibling;
			this.currentElementFinished = false;
		}
	}
	
	if (theSkipFlag)
	{
		theResult = this.nextElement();
	}
	
	return theResult;
}

GBWebPageIterator.prototype.nextPortionOfElements = function(anAmount)
{
	var theElementsIterationResult = this.nextElement();
	var theResult = theElementsIterationResult;
	
	for (var thePortion = 0; thePortion < anAmount; thePortion ++)
	{
		if (theElementsIterationResult == "endOfElementsIteration")
		{
			break;
		}

		theElementsIterationResult = this.nextElement();
		theResult +=  this.outputDivider + theElementsIterationResult;
	}
	
	return theResult;
}

//////////////////////////////////////////////////////////////////////////
// Head meta elements extraction
GBWebPageIterator.prototype.documentHead = function()
{
	var theHeadElement = null;
	var theDocumentChildren = this.currentDocument.documentElement.children;
	for (var theIndex = 0; theIndex < theDocumentChildren.length; theIndex ++)
	{
		var theElement = theDocumentChildren[theIndex];
		if ("head" == theElement.tagName.toLowerCase())
		{
			theHeadElement = theElement;
			break;
		}
	}
	
	var theResult = "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">\n";
	if (!theHeadElement)
	{
		return theResult;
	}
	
	var theHeadChildren = theHeadElement.children;
	for (var theIndex = 0; theIndex < theHeadChildren.length; theIndex ++)
	{
		var theHeadPart = theHeadChildren[theIndex];
		var theHeadPartTagName = theHeadPart.tagName.toLowerCase();
		// Process only title and meta tags
		if ("title" == theHeadPartTagName || "meta" == theHeadPartTagName)
		{
			// Skip content type declaration
			var theHTTPEquiv = theHeadPart.httpEquiv;
			if (!theHTTPEquiv || theHTTPEquiv.toLowerCase() != "content-type")
			{
				theResult += theHeadPart.outerHTML + "\n";
			}	
		}
	}

	return theResult;
}

//////////////////////////////////////////////////////////////////////////
// StyleSheets iteration
GBWebPageIterator.prototype.formatStyleSheet = function(aStyleSheet)
{
	var theMedia = "nil";
	var theHref = "nil";
	if (aStyleSheet)
	{
		theMedia = (aStyleSheet.media && aStyleSheet.media.mediaText && 0 != aStyleSheet.media.mediaText) ? 
					aStyleSheet.media.mediaText : "undefined"
		theHref = aStyleSheet.href ? aStyleSheet.href : "undefined";
	}

	var theRules = aStyleSheet.cssRules;
	var theProcessFlag = theRules ? "yes" : "no";
	return "beginStyleSheet:" + this.dataPartsDivier + theMedia + 
				this.dataPartsDivier + theHref + this.dataPartsDivier + 
				theProcessFlag;
}

GBWebPageIterator.prototype.formatStyleSheetRule = function(aStyleSheetRule)
{
	var theRuleRepresentation = "";
	var theChildStyleSheet = aStyleSheetRule.styleSheet;
	if (theChildStyleSheet)
	{
		var theChildRules = theChildStyleSheet.cssRules;
		if (theChildRules)
		{
			var theChildRulesCount = theChildRules.length;
			
			for (var i = 0; i < theChildRulesCount; i++)
			{
				var theChildRuleRepresentation = this.formatStyleSheetRule(theChildRules[i]);
				theRuleRepresentation += theChildRuleRepresentation;
				
				if (i < theChildRulesCount - 1)
				{
					theRuleRepresentation += this.outputDivider;
				}
			}
		}
	}
	
	if (theRuleRepresentation.length > 0)
	{
		return theRuleRepresentation;
	}
	
	return "beginStyleSheetRule:" + this.dataPartsDivier + (aStyleSheetRule ? aStyleSheetRule.cssText : "nil");
}

GBWebPageIterator.prototype.beginStylesIteration = function()
{
	this.currentStyleSheets = this.currentDocument.styleSheets;
	this.currentStyleSheetIndex = 0;
	this.currentRuleIndex = 0;
	
	if (0 == this.currentStyleSheets.length)
	{
		return "endStylesIteration";
	}
	
	return this.formatStyleSheet(this.currentStyleSheets[this.currentStyleSheetIndex]);
}

GBWebPageIterator.prototype.nextStyleRule = function()
{
	if (this.currentStyleSheetIndex >= this.currentStyleSheets.length)
	{
		return "endStylesIteration";
	}

	var theRules = this.currentStyleSheets[this.currentStyleSheetIndex].cssRules;
		
	if ((!theRules || this.currentRuleIndex >= theRules.length) && 
				this.currentRuleIndex != -1)
	{
		this.currentRuleIndex = -1;
		return "endStyleSheet";
	}
	
	if (-1 == this.currentRuleIndex)
	{
		this.currentStyleSheetIndex ++;
		this.currentRuleIndex = 0;
		if (this.currentStyleSheetIndex >= this.currentStyleSheets.length)
		{
			return "endStylesIteration";
		}
		else
		{
			return this.formatStyleSheet(this.currentStyleSheets[this.currentStyleSheetIndex]);
		}
	}
	
	return this.formatStyleSheetRule(theRules[this.currentRuleIndex ++]);
}

GBWebPageIterator.prototype.nextPortionOfStyleRules = function(anAmount)
{
	var theStyleIterationResult = this.nextStyleRule();
	var theResult = theStyleIterationResult;
	
	for (var thePortion = 0; thePortion < anAmount; thePortion ++)
	{
		if (theStyleIterationResult == "endStylesIteration" || 
					theStyleIterationResult == "endStyleSheet")
		{
			break;
		}

		theStyleIterationResult = this.nextStyleRule();
		theResult +=  this.outputDivider + theStyleIterationResult;
	}
	
	return theResult;
}

//////////////////////////////////////////////////////////////////////////
var gGBWebPageIterator = new GBWebPageIterator();
//////////////////////////////////////////////////////////////////////////
